@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2009 SubDownloader Developers - See COPYING - GPLv3
+# Copyright (c) 2010 SubDownloader Developers - See COPYING - GPLv3
 
 """ Create and launch the GUI """
 import sys, re, os, traceback, tempfile, time, thread, webbrowser, urllib2, base64, zlib, commands, platform, os.path, zipfile, __builtin__, gettext, locale
@@ -43,6 +43,8 @@ from FileManagement import FileScan, Subtitle
 from modules.videofile import  *
 from modules.subtitlefile import *
 from modules.search import *
+import modules.OSHttpRequests as OSHttpRequests
+
 import modules.utils as utils
 import languages.Languages as languages
 
@@ -234,9 +236,9 @@ class Main(QObject, Ui_MainWindow):
         self.donate_button = QtGui.QPushButton("   " + _("Help Us With 5 USD/EUR"))
         #self.donate_button.setIndent(10)
 
-        iconpaypal = QtGui.QIcon()
-        iconpaypal.addPixmap(QtGui.QPixmap(":/images/paypal.png"), QtGui.QIcon.Normal, QtGui.QIcon.On)
         if platform.system() in ("Windows", "Microsoft"):
+            iconpaypal = QtGui.QIcon()
+            iconpaypal.addPixmap(QtGui.QPixmap(":/images/paypal.png"), QtGui.QIcon.Normal, QtGui.QIcon.On)
             self.donate_button.setIcon(iconpaypal)
             self.donate_button.setIconSize(QtCore.QSize(50, 24))
 
@@ -253,7 +255,7 @@ class Main(QObject, Ui_MainWindow):
             #print self.OSDBServer.xmlrpc_server.GetTranslation(self.OSDBServer._token, 'ar', 'po','subdownloader')
             self.window.setCursor(Qt.WaitCursor)
             if self.establishServerConnection():# and self.OSDBServer.is_connected():
-                thread.start_new_thread(self.update_users, (300, )) #update the users counter every 5min
+                #thread.start_new_thread(self.update_users, (300, )) #update the users counter every 5min
                 if platform.system() in ("Windows", "Microsoft"):
                     thread.start_new_thread(self.detect_software_updates, ())
                 if SHAREWARE:
@@ -274,7 +276,7 @@ class Main(QObject, Ui_MainWindow):
                     loginUsername = str(settings.value("options/LoginUsername", QVariant()).toString().toUtf8())
                     loginPassword = str(settings.value("options/LoginPassword", QVariant()).toString().toUtf8())
                 self.login_user(loginUsername,loginPassword,self.window)
-                thread.start_new_thread(self.OSDBServer.NoOperation, (900, )) #check expire session every 15min
+                #thread.start_new_thread(self.OSDBServer.NoOperation, (900, )) #check expire session every 15min
             else:
                 QMessageBox.about(self.window,_("Error"),_("Error contacting the server. Please try again later"))
             self.window.setCursor(Qt.ArrowCursor)
@@ -1019,6 +1021,7 @@ class Main(QObject, Ui_MainWindow):
 
     def onButtonDownload(self):
         #We download the subtitle in the same folder than the video
+	    osHttpRequests = OSHttpRequests.OSHttpRequests()
             subs = self.videoModel.getCheckedSubtitles()
             replace_all  = False
             skip_all = False
@@ -1119,10 +1122,12 @@ class Main(QObject, Ui_MainWindow):
                 try:
                    if not skip_all:
                         log.debug("Downloading subtitle '%s'" % destinationPath)
-                        if self.OSDBServer.DownloadSubtitles({sub.getIdFileOnline():destinationPath}):
-                            success_downloaded += 1
-                        else:
-                            QMessageBox.about(self.window,_("Error"),_("Unable to download subtitle %s") %sub.getFileName())
+			#print {sub.getIdFileOnline():destinationPath}
+                        osHttpRequests.download('http://www.opensubtitles.org/en/download/file/%s.gz' %sub.getIdFileOnline(), destinationPath)
+                        #if self.OSDBServer.DownloadSubtitles({sub.getIdFileOnline():destinationPath}):
+                            #success_downloaded += 1
+                        #else:
+                            #QMessageBox.about(self.window,_("Error"),_("Unable to download subtitle %s") %sub.getFileName())
                 except Exception, e:
                     traceback.print_exc(e)
                     QMessageBox.about(self.window,_("Error"),_("Unable to download subtitle %s") % sub.getFileName())
@@ -1132,18 +1137,8 @@ class Main(QObject, Ui_MainWindow):
             self.progress(100)
 
     def showErrorConnection(self):
-        warningBox = QMessageBox(_("Error write permission"),
-                                    _("%s cannot be saved.\nCheck that the folder exists and you have write-access permissions.") %destinationPath ,
-                                    QMessageBox.Warning,
-                                    QMessageBox.Retry | QMessageBox.Default ,
-                                    QMessageBox.Discard | QMessageBox.Escape,
-                                    QMessageBox.NoButton,
-                                    self.window)
-        answer = warningBox.exec_()
-        if answer == QMessageBox.Retry:
-            pass #Try to create connection + login
-        elif answer == QMessageBox.Discard :
-            return
+	QMessageBox.about(self.window,_("Alert"),_("www.opensubtitles.org is not responding\nIt might be overloaded, try again in a few moments."))
+
 
     """Control the STATUS BAR PROGRESS"""
     def progress(self, val = None,msg = None):
