@@ -1,46 +1,62 @@
 #!/usr/bin/env python
-# Copyright (c) 2009 SubDownloader Developers - See COPYING - GPLv3
+# -*- coding: utf-8 -*-
+# Copyright (c) 2010 SubDownloader Developers - See COPYING - GPLv3
 
 from xmlrpclib import Transport,ServerProxy,ProtocolError
 import base64, httplib, os
 import StringIO, gzip, zlib
 import logging
 import threading, thread
+import sys
 log = logging.getLogger("subdownloader.WebService")
     
 from modules import APP_TITLE, APP_VERSION
 import modules.videofile as videofile
 import modules.subtitlefile as subtitlefile
 from FileManagement import Subtitle
+import socket, urllib2
+
 
 DEFAULT_OSDB_SERVER = "http://api.opensubtitles.org/xml-rpc"
 DEFAULT_SDDB_SERVER = "http://sddb.subdownloader.net/xmlrpc/"
 DEFAULT_PROXY = 'http://w2.hidemyass.com/'
+TEST_URL = 'http://www.google.com'
 USER_AGENT = "%s %s"% (APP_TITLE, APP_VERSION)
+CON_TIMEOUT = 300
 
-def test_connection(url, timeout=15):
-    import socket, urllib2
+def test_connection(url, timeout=CON_TIMEOUT):
     defTimeOut=socket.getdefaulttimeout()
     socket.setdefaulttimeout(timeout)
-    connectable=True
+    connectable=False
     try:
-        urllib2.urlopen(url)
-    except (urllib2.HTTPError, urllib2.URLError, socket.error, socket.sslerror):
-        connectable=False
+       	urllib2.urlopen(url)
+	log.debug("successfully tested connection")
+	connectable=True
+    except urllib2.HTTPError, e:
+       	log.error('The server couldn\'t fulfill the request. Error code: '% e.code)
+    except urllib2.URLError, e:
+       	log.error('We failed to reach a server. Reason: %s '% e.reason)
+    except socket.error, (value,message):
+       	log.error("Could not open socket: %s"% message)
+    except socket.sslerror, (value,message):
+       	log.error("Could not open ssl socket: %s"% message)
     socket.setdefaulttimeout(defTimeOut)
     return connectable
 
+
 class TimeoutFunctionException(Exception): 
     """Exception to raise on a timeout""" 
     pass 
 
 class TimeoutFunction: 
 
-    def __init__(self, function, timeout=30): 
+    def __init__(self, function, timeout=CON_TIMEOUT): 
+	self.log = logging.getLogger("subdownloader.SDService.TimeoutFunction")
         self.timeout = timeout 
         self.function = function 
 
     def handle_timeout(self): 
+	self.log.debug("exception in timeouted function %s"%self.function)
         raise TimeoutFunctionException()
 
     def __call__(self, *args): 
@@ -50,6 +66,8 @@ class TimeoutFunction:
         try: 
             t.start()
             result = self.function(*args)
+	except Exception, e:
+	    raise e
         finally: 
             #signal.signal(signal.SIGALRM, old)
             t.cancel()
@@ -63,13 +81,13 @@ class ProxiedTransport(Transport):
     """ Used for proxied connections to the XMLRPC server
     """
     def __init__(self):
-        #self.log = logging.getLogger("subdownloader.OSDBServer.ProxiedTransport")
+        self.log = logging.getLogger("subdownloader.OSDBServer.ProxiedTransport")
         self._use_datetime = True # annoying -> AttributeError: Main instance has no attribute '_use_datetime'
     def set_proxy(self, proxy):
         self.proxy = proxy
-        #self.log.debug("Proxy set to: %s"% proxy)
+        self.log.debug("Proxy set to: %s"% proxy)
     def make_connection(self, host):
-        #self.log.debug("Connecting to %s through %s"% (host, self.proxy))
+        self.log.debug("Connecting to %s through %s"% (host, self.proxy))
         self.realhost = host
         h = httplib.HTTP(self.proxy)
         return h
@@ -118,13 +136,14 @@ class SDService(object):
             raise e
             
     def create_xmlrpcserver(self, server, proxy):
-        self.log.debug("Creating XMLRPC server connection...")
+        self.log.debug("Creating XMLRPC server connection... to server %s with proxy %s" %(server,proxy))
         try:
             return self.connect(server, proxy)
         except Exception, e:
             raise e
         
     def connect(self, server, proxy):
+	connect=False
         try:
             self.log.debug("Connecting with parameters (%r, %r)" %(server, proxy))
             connect = TimeoutFunction(self._connect)
@@ -132,9 +151,12 @@ class SDService(object):
         except TimeoutFunctionException, e:
             self.log.error("Connection timed out. Maybe you need a proxy.")
             raise e
-        #except Exception, e:
-            #import sys
-            #self.log.error("Unexpected error: %s", sys.exc_info())
+        except Exception, e:
+            self.log.error("connect: Unexpected error: %s", sys.exc_info())
+	    raise e
+	finally:
+	    self.log.debug("connection connected %s"%connect)
+	    return connect
 
     def _connect(self, server, proxy):
         try:
@@ -147,7 +169,7 @@ class SDService(object):
                 self.log.debug("...connected")
                 return True
                 
-            elif test_connection(server):
+            elif test_connection(TEST_URL):
                     self.log.debug("Trying direct connection...")
                     self.xmlrpc_server = ServerProxy(server, allow_none=True)
                     #self.ServerInfo()
@@ -157,9 +179,15 @@ class SDService(object):
                 self.log.debug("...failed")
                 self.log.error("Unable to connect. Try setting a proxy.")
                 return False
+	except xmlrpclib.ProtocolError, e:
+		self.log.debug("error in HTTP/HTTPS transport layer")
+		raise e
+	except xmlrpclib.Fault, e:
+		self.log.debug("error in xml-rpc server")
+		raise e
         except Exception,e:
-                    self.log.debug("Connection to the server failed")
-                    raise e
+                self.log.debug("Connection to the server failed/other error:%s",sys.exc_info())
+                raise e
         
     def is_connected(self):
         """ 
@@ -170,7 +198,7 @@ class SDService(object):
         return self._token != None
         
     def ServerInfo(self):
-        ServerInfo = TimeoutFunction(self._ServerInfo)
+        ServerInfo = self._ServerInfo
         try:
             a = ServerInfo()
             return a
@@ -178,9 +206,9 @@ class SDService(object):
             self.log.error("ServerInfo timed out")
             
         except Exception, e:
-            #print type(e)     # the exception instance
-            #print e.args      # arguments stored in .args
-            #print e           # __str__ allows args to printed directly
+            print type(e)     # the exception instance
+            print e.args      # arguments stored in .args
+            print e           # __str__ allows args to printed directly
             self.log.error("ServerInfo error connection.")
             raise e
 
@@ -193,11 +221,13 @@ class SDService(object):
             raise
             
     def login(self, username="", password=""):
-        login = TimeoutFunction(self._login)
         try:
+	    login = TimeoutFunction(self._login)
             return login(username, password)
         except TimeoutFunctionException:
             self.log.error("login timed out")
+	except Exception, e:
+	    self.log.error("login: other issue:%s",sys.exc_info()[0])
         
     def _login(self, username="", password=""):
         """Login to the Server using username/password,
@@ -206,8 +236,14 @@ class SDService(object):
         """ 
         self.log.debug("----------------")
         self.log.debug("Logging in (username: %s)..."% username)
-        info = self.xmlrpc_server.LogIn(username, password, self.language, self.user_agent)
-        self.log.debug("Login ended in %s with status: %s"% (info['seconds'], info['status']))
+	try: 
+	        info = self.xmlrpc_server.LogIn(username, password, self.language, self.user_agent)
+	        self.log.debug("Login ended in %s with status: %s"% (info['seconds'], info['status']))
+	except : 
+		self.log.debug("Unexpected error: %s", sys.exc_info()[0])
+		self._token = None
+		return False
+        
         if info['status'] == "200 OK":
             self.log.debug("Session ID: %s"% info['token'])
             self.log.debug("----------------")
@@ -220,8 +256,8 @@ class SDService(object):
             return False
         
     def logout(self):
-        logout = TimeoutFunction(self._logout)
         try:
+	    logout = TimeoutFunction(self._logout)
             return logout()
         except TimeoutFunctionException:
             self.log.error("logout timed out")
@@ -231,11 +267,21 @@ class SDService(object):
         This functions doesn't return any boolean value, since it can 'fail' for anonymous logins
         """
         self.log.debug("Logging out from session ID: %s"% self._token)
-        info = self.xmlrpc_server.LogOut(self._token)
-        self.log.debug("Logout ended in %s with status: %s"% (info['seconds'], info['status']))
-        # force token reset
-        self._token = None
-        
+	try:
+		info = self.xmlrpc_server.LogOut(self._token)
+                self.log.debug("Logout ended in %s with status: %s"% (info['seconds'], info['status']))
+        except xmlrpclib.ProtocolError, e:
+                self.log.debug("error in HTTP/HTTPS transport layer")
+                raise e
+        except xmlrpclib.Fault, e:
+                self.log.debug("error in xml-rpc server")
+                raise e
+        except:
+                self.log.debug("Connection to the server failed/other error:%s",sys.exc_info())
+                raise e
+	finally:
+        	# force token reset
+	        self._token = None
         
     #
     # SUBTITLE METHODS 
@@ -246,6 +292,8 @@ class SDService(object):
             return GetSubLanguages(language)
         except TimeoutFunctionException:
             self.log.error("GetSubLanguages timed out")
+	except:
+	    self.log.error("GetSubLanguages other error")
             
     def _GetSubLanguages(self, language):
         """Return all suported subtitles languages in a dictionary
@@ -256,14 +304,22 @@ class SDService(object):
         if language == "all":
             # return result right away if no 'translation' needed
             return "all"
-        info = self.xmlrpc_server.GetSubLanguages(language)
-        self.log.debug("GetSubLanguages complete in %s"% info['seconds'])
-            
-        if language:
-            for lang in info['data']:
-                if lang['ISO639'] == language: return lang['SubLanguageID']
-                
-        return info['data']
+	try:
+	        info = self.xmlrpc_server.GetSubLanguages(language)
+        	self.log.debug("GetSubLanguages complete in %s"% info['seconds'])
+                if language:
+                    for lang in info['data']:
+                        if lang['ISO639'] == language: return lang['SubLanguageID']
+                return info['data']
+	except xmlrpclib.ProtocolError, e:
+                self.log.debug("error in HTTP/HTTPS transport layer")
+                raise e
+        except xmlrpclib.Fault, e:
+                self.log.debug("error in xml-rpc server")
+                raise e
+        except:
+                self.log.debug("Connection to the server failed/other error:%s",sys.exc_info())
+                raise e
         
     def CheckSubHash(self, hashes):
         CheckSubHash = TimeoutFunction(self._CheckSubHash)
@@ -271,6 +327,8 @@ class SDService(object):
             return CheckSubHash(hashes)
         except TimeoutFunctionException:
             self.log.error("CheckSubHash timed out")
+	except:
+	    self.log.error("CheckSubHash other error")
         
     def _CheckSubHash(self, hashes):
         """
@@ -287,14 +345,25 @@ class SDService(object):
             for sub in video.getSubtitles():
                 hashes.append(sub.getHash())
             self.log.debug("...done")
-        info = self.xmlrpc_server.CheckSubHash(self._token,hashes)
-        self.log.debug("CheckSubHash ended in %s with status: %s"% (info['seconds'], info['status']))
-        result = {}
-        for hash in hashes:
-            result[hash] = info['data'][hash]
-#        for data in info['data']:
-#            result[data.key()] = data.value()
-        return result
+	try:
+	        info = self.xmlrpc_server.CheckSubHash(self._token,hashes)
+        	self.log.debug("CheckSubHash ended in %s with status: %s"% (info['seconds'], info['status']))
+	        result = {}
+	        for hash in hashes:
+        	    result[hash] = info['data'][hash]
+	#        for data in info['data']:
+#       	     result[data.key()] = data.value()
+	        return result
+        except xmlrpclib.ProtocolError, e:
+                self.log.debug("error in HTTP/HTTPS transport layer")
+                raise e
+        except xmlrpclib.Fault, e:
+                self.log.debug("error in xml-rpc server")
+                raise e
+        except:
+                self.log.debug("Connection to the server failed/other error:%s",sys.exc_info())
+                raise e
+
     
     def DownloadSubtitles(self, subtitles):
         DownloadSubtitles = TimeoutFunction(self._DownloadSubtitles)
@@ -321,21 +390,32 @@ class SDService(object):
         if len(subtitles_to_download):
             self.log.debug("Communicating with server...")
             self.log.debug("xmlrpc_server.DownloadSubtitles(%s,%r)" %(self._token, subtitles_to_download.keys()))
-            answer = self.xmlrpc_server.DownloadSubtitles(self._token, subtitles_to_download.keys())
-            self.log.debug("DownloadSubtitles finished in %s with status %s."% (answer['seconds'], answer['status']))
-            
-            if answer.has_key("data"):
-                if answer['data'] == False:  #TODO support passing the reason of the erorr to be shown in the GUI
-                    self.log.info("Error downloading subtitle.")
+	    try:
+		answer = self.xmlrpc_server.DownloadSubtitles(self._token, subtitles_to_download.keys())
+		self.log.debug("DownloadSubtitles finished in %s with status %s."% (answer['seconds'], answer['status']))
+	    except xmlrpclib.ProtocolError, e:
+                self.log.debug("error in HTTP/HTTPS transport layer")
+               	raise e
+	    except xmlrpclib.Fault, e:
+                self.log.debug("error in xml-rpc server")
+               	raise e
+	    except:
+                self.log.debug("Connection to the server failed/other error:%s",sys.exc_info())
+               	raise e
+	    else:
+                if answer.has_key("data"):
+                    if answer['data'] == False:  #TODO support passing the reason of the erorr to be shown in the GUI
+                       self.log.info("Error downloading subtitle.")
+                       return False
+                    self.log.debug("Got %i subtitles from server. Uncompressing data..."% len(answer['data']))
+                    for sub in answer['data']:
+                        #print subtitles_to_download[sub['idsubtitlefile']]['subtitle_path']
+                        #print subtitles_to_download[sub['idsubtitlefile']]['video'].getFileName()
+                        self.BaseToFile(sub['data'], subtitles_to_download[sub['idsubtitlefile']])
+                    return answer['data']
+                else:
+                    self.log.info("Server sent no subtitles to me.")
                     return False
-                self.log.debug("Got %i subtitles from server. Uncompressing data..."% len(answer['data']))
-                for sub in answer['data']:
-                    #self.log.info("%s -> %s"% (subtitles_to_download[sub['idsubtitlefile']]['subtitle_path'], subtitles_to_download[sub['idsubtitlefile']]['video'].getFileName()))
-                    self.BaseToFile(sub['data'], subtitles_to_download[sub['idsubtitlefile']])
-                return answer['data']
-            else:
-                self.log.info("Server sent no subtitles to me.")
-                return False
         
     def SearchSubtitles(self, language="all", videos=None, imdb_ids=None):
         SearchSubtitles = TimeoutFunction(self._SearchSubtitles)
@@ -652,6 +732,8 @@ class SDService(object):
             return CheckSoftwareUpdates(app)
         except TimeoutFunctionException:
             self.log.error("CheckSoftwareUpdates timed out")
+	except: 
+	    self.log.error("CheckSoftwareUpdates other error")
         
     def _CheckSoftwareUpdates(self, app=None):
         """Returns latest info on the given application if available
@@ -659,18 +741,30 @@ class SDService(object):
         self.log.debug("----------------")
         self.log.debug("CheckSoftwareUpdates RPC method starting...")
         if not app: app = APP_TITLE.lower()
-        info = self.xmlrpc_server.CheckSoftwareUpdates(app)
-
-        # we have something to show
-        self.log.debug("Latest SubDownloader Version Found: %s"% info['latest_version'])
-        return info
+	try:
+	        info = self.xmlrpc_server.CheckSoftwareUpdates(app)
+	except xmlrpclib.ProtocolError, e:
+                self.log.debug("error in HTTP/HTTPS transport layer")
+                raise e
+        except xmlrpclib.Fault, e: 
+                self.log.debug("error in xml-rpc server")
+                raise e
+        except Exception,e:
+                self.log.debug("Connection to the server failed/other error:%s",sys.exc_info())
+                raise e
+	else:
+        	# we have something to show
+        	self.log.debug("Latest SubDownloader Version Found: %s"% info['latest_version'])
+        	return info
         
     def NoOperation(self, token=None):
-        NoOperation = TimeoutFunction(self._NoOperation)
         try:
+	    NoOperation = TimeoutFunction(self._NoOperation)
             return NoOperation()
         except TimeoutFunctionException:
             self.log.error("NoOperation timed out")
+	except:
+	    self.log.error("NoOperation other error")
         
     def _NoOperation(self):
         """This method should be called every 15 minutes after last request to xmlrpc. 
@@ -679,12 +773,24 @@ class SDService(object):
         """
         self.log.debug("----------------")
         self.log.debug("NoOperation RPC method starting...")
-        info = self.xmlrpc_server.NoOperation(self._token)
-        self.log.debug("NoOperation finished in %s with status %s."% (info['seconds'], info['status']))
-        self.log.debug("----------------")
-        if info['status'] != "200 OK":
-            return False
-        return True
+	noop=False
+	try:
+        	info = self.xmlrpc_server.NoOperation(self._token)
+	        self.log.debug("NoOperation finished in %s with status %s."% (info['seconds'], info['status']))
+                self.log.debug("----------------")
+                if info['status'] == "200 OK":
+                	return True
+		else:
+			return noop
+        except xmlrpclib.ProtocolError, e:
+                self.log.debug("error in HTTP/HTTPS transport layer")
+                raise e
+        except xmlrpclib.Fault, e:
+                self.log.debug("error in xml-rpc server")
+                raise e
+        except:
+                self.log.debug("Connection to the server failed/other error:%s",sys.exc_info())
+                raise e
         
     def SearchToMail(self, videos, languages):
         SearchToMail = TimeoutFunction(self._SearchToMail)
@@ -704,8 +810,18 @@ class SDService(object):
         for video in videos:
             array = {'moviehash': video.getHash(), 'moviesize': video.getSize()}
             video_array.append(array)
-        info = self.xmlrpc_server.SearchToMail(self._token, languages, video_array)
-        self.log.debug("SearchToMail finished in %s with status %s."% (info['seconds'], info['status']))
+	try:
+	        info = self.xmlrpc_server.SearchToMail(self._token, languages, video_array)
+        	self.log.debug("SearchToMail finished in %s with status %s."% (info['seconds'], info['status']))
+	except xmlrpclib.ProtocolError, e:
+                self.log.debug("error in HTTP/HTTPS transport layer")
+                raise e
+        except xmlrpclib.Fault, e:
+                self.log.debug("error in xml-rpc server")
+                raise e
+        except:
+                self.log.debug("Connection to the server failed/other error:%s",sys.exc_info())
+                raise e
         
     def BaseToFile(self, base_data, path):
         """This will decode the base64 data and save it as a file with the given path
